@@ -245,6 +245,44 @@ static enum connman_service_error string2error(const char *error)
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
 }
 
+static connman_bool_t is_connecting(struct connman_service *service)
+{
+	switch (service->state) {
+	case CONNMAN_SERVICE_STATE_UNKNOWN:
+	case CONNMAN_SERVICE_STATE_IDLE:
+	case CONNMAN_SERVICE_STATE_FAILURE:
+	case CONNMAN_SERVICE_STATE_DISCONNECT:
+	case CONNMAN_SERVICE_STATE_READY:
+	case CONNMAN_SERVICE_STATE_LOGIN:
+	case CONNMAN_SERVICE_STATE_ONLINE:
+		break;
+	case CONNMAN_SERVICE_STATE_ASSOCIATION:
+	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static connman_bool_t is_connected(const struct connman_service *service)
+{
+	switch (service->state) {
+	case CONNMAN_SERVICE_STATE_UNKNOWN:
+	case CONNMAN_SERVICE_STATE_IDLE:
+	case CONNMAN_SERVICE_STATE_ASSOCIATION:
+	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+	case CONNMAN_SERVICE_STATE_DISCONNECT:
+	case CONNMAN_SERVICE_STATE_FAILURE:
+		break;
+	case CONNMAN_SERVICE_STATE_READY:
+	case CONNMAN_SERVICE_STATE_LOGIN:
+	case CONNMAN_SERVICE_STATE_ONLINE:
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static struct connman_service *get_default(void)
 {
 	struct connman_service *service;
@@ -257,7 +295,7 @@ static struct connman_service *get_default(void)
 
 	service = g_sequence_get(iter);
 
-	if (service->state != CONNMAN_SERVICE_STATE_READY)
+	if (is_connected(service) == FALSE)
 		return NULL;
 
 	return service;
@@ -425,19 +463,9 @@ static void append_ethernet(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
 
-	switch (service->state) {
-	case CONNMAN_SERVICE_STATE_UNKNOWN:
-	case CONNMAN_SERVICE_STATE_IDLE:
-	case CONNMAN_SERVICE_STATE_FAILURE:
-	case CONNMAN_SERVICE_STATE_DISCONNECT:
+	if (is_connecting(service) == FALSE &&
+			is_connected(service) == FALSE)
 		return;
-	case CONNMAN_SERVICE_STATE_ASSOCIATION:
-	case CONNMAN_SERVICE_STATE_CONFIGURATION:
-	case CONNMAN_SERVICE_STATE_READY:
-	case CONNMAN_SERVICE_STATE_LOGIN:
-	case CONNMAN_SERVICE_STATE_ONLINE:
-		break;
-	}
 
 	if (service->ipconfig != NULL)
 		__connman_ipconfig_append_ethernet(service->ipconfig, iter);
@@ -447,19 +475,8 @@ static void append_ipv4(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
 
-	switch (service->state) {
-	case CONNMAN_SERVICE_STATE_UNKNOWN:
-	case CONNMAN_SERVICE_STATE_IDLE:
-	case CONNMAN_SERVICE_STATE_FAILURE:
-	case CONNMAN_SERVICE_STATE_DISCONNECT:
-	case CONNMAN_SERVICE_STATE_ASSOCIATION:
-	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+	if (is_connected(service) == FALSE)
 		return;
-	case CONNMAN_SERVICE_STATE_READY:
-	case CONNMAN_SERVICE_STATE_LOGIN:
-	case CONNMAN_SERVICE_STATE_ONLINE:
-		break;
-	}
 
 	if (service->ipconfig != NULL)
 		__connman_ipconfig_append_ipv4(service->ipconfig, iter);
@@ -477,19 +494,8 @@ static void append_proxy(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
 
-	switch (service->state) {
-	case CONNMAN_SERVICE_STATE_UNKNOWN:
-	case CONNMAN_SERVICE_STATE_IDLE:
-	case CONNMAN_SERVICE_STATE_FAILURE:
-	case CONNMAN_SERVICE_STATE_DISCONNECT:
-	case CONNMAN_SERVICE_STATE_ASSOCIATION:
-	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+	if (is_connected(service) == FALSE)
 		return;
-	case CONNMAN_SERVICE_STATE_READY:
-	case CONNMAN_SERVICE_STATE_LOGIN:
-	case CONNMAN_SERVICE_STATE_ONLINE:
-		break;
-	}
 
 	if (service->ipconfig != NULL)
 		__connman_ipconfig_append_proxy(service->ipconfig, iter);
@@ -844,25 +850,6 @@ static DBusMessage *clear_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static connman_bool_t is_connecting(struct connman_service *service)
-{
-	switch (service->state) {
-	case CONNMAN_SERVICE_STATE_UNKNOWN:
-	case CONNMAN_SERVICE_STATE_IDLE:
-	case CONNMAN_SERVICE_STATE_FAILURE:
-	case CONNMAN_SERVICE_STATE_DISCONNECT:
-	case CONNMAN_SERVICE_STATE_READY:
-	case CONNMAN_SERVICE_STATE_LOGIN:
-	case CONNMAN_SERVICE_STATE_ONLINE:
-		break;
-	case CONNMAN_SERVICE_STATE_ASSOCIATION:
-	case CONNMAN_SERVICE_STATE_CONFIGURATION:
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
 static connman_bool_t is_ignore(struct connman_service *service)
 {
 	if (service->autoconnect == FALSE)
@@ -901,7 +888,7 @@ void __connman_service_auto_connect(void)
 		if (service->favorite == FALSE)
 			return;
 
-		if (service->state == CONNMAN_SERVICE_STATE_READY)
+		if (is_connected(service) == TRUE)
 			return;
 
 		if (is_ignore(service) == FALSE &&
@@ -1012,6 +999,64 @@ static connman_bool_t get_reconnect_state(struct connman_service *service)
 	return __connman_device_get_reconnect(device);
 }
 
+struct connman_service *
+__connman_service_connect_type(enum connman_service_type type)
+{
+	struct connman_service *service;
+	GSequenceIter *iter;
+	int err;
+
+	DBG("type %d", type);
+
+	/*
+	 * We go through the already sorted service list.
+	 * We pick the first one matching our type, or just
+	 * the first available one if we have no type.
+	 */
+	iter = g_sequence_get_begin_iter(service_list);
+	service = g_sequence_get(iter);
+
+	/*
+	 * If the first service is connected or about to be
+	 * connected, we return it, regardless of the type.
+	 */
+	if ((g_sequence_iter_is_end(iter) == FALSE) &&
+		(is_connecting(service) == TRUE ||
+			is_connected(service) == TRUE))
+		return service;
+
+	while (g_sequence_iter_is_end(iter) == FALSE) {
+		if (service->type == type ||
+			type == CONNMAN_SERVICE_TYPE_UNKNOWN)
+			break;
+
+		iter = g_sequence_iter_next(iter);
+		service = g_sequence_get(iter);
+	}
+
+	if (g_sequence_iter_is_end(iter))
+		return NULL;
+
+	service->ignore = FALSE;
+
+	service->userconnect = TRUE;
+
+	set_reconnect_state(service, FALSE);
+
+	err = __connman_service_connect(service);
+	if (err < 0) {
+		if (err == -ENOKEY)
+			if (__connman_agent_request_passphrase(service,
+								NULL, NULL))
+				return service;
+
+		if (err != -EINPROGRESS)
+			return NULL;
+	}
+
+	return service;
+}
+
 static DBusMessage *connect_service(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
@@ -1361,9 +1406,9 @@ static gint service_compare(gconstpointer a, gconstpointer b,
 	struct connman_service *service_b = (void *) b;
 
 	if (service_a->state != service_b->state) {
-		if (service_a->state == CONNMAN_SERVICE_STATE_READY)
+		if (is_connected(service_a) == TRUE)
 			return -1;
-		if (service_b->state == CONNMAN_SERVICE_STATE_READY)
+		if (is_connected(service_b) == TRUE)
 			return 1;
 	}
 
@@ -1573,7 +1618,8 @@ int __connman_service_indicate_state(struct connman_service *service,
 			__connman_service_auto_connect();
 	}
 
-	if (state == CONNMAN_SERVICE_STATE_READY) {
+	if (state == CONNMAN_SERVICE_STATE_READY ||
+			state == CONNMAN_SERVICE_STATE_ONLINE) {
 		set_reconnect_state(service, TRUE);
 
 		__connman_service_set_favorite(service, TRUE);
@@ -1724,7 +1770,7 @@ int __connman_service_connect(struct connman_service *service)
 
 	DBG("service %p", service);
 
-	if (service->state == CONNMAN_SERVICE_STATE_READY)
+	if (is_connected(service) == TRUE)
 		return -EISCONN;
 
 	if (is_connecting(service) == TRUE)
@@ -1902,7 +1948,7 @@ int __connman_service_create_and_connect(DBusMessage *msg)
 	struct connman_network *network;
 	struct connman_device *device;
 	DBusMessageIter iter, array;
-	const char *mode = "managed", *security = "none";
+	const char *mode = "managed", *security = "none", *group_security;
 	const char *type = NULL, *ssid = NULL, *passphrase = NULL;
 	unsigned int ssid_len = 0;
 	const char *ident;
@@ -1973,8 +2019,15 @@ int __connman_service_create_and_connect(DBusMessage *msg)
 	if (ident == NULL)
 		return -EOPNOTSUPP;
 
+
+	if (!g_strcmp0(security, "wpa") ||
+		!g_strcmp0(security, "rsn"))
+		group_security = "psk";
+	else
+		group_security = security;
+
 	group = connman_wifi_build_group_name((unsigned char *) ssid,
-						ssid_len, mode, security);
+						ssid_len, mode, group_security);
 	if (group == NULL)
 		return -EINVAL;
 
@@ -2173,10 +2226,24 @@ static void setup_ipconfig(struct connman_service *service, int index)
 void __connman_service_create_ipconfig(struct connman_service *service,
 								int index)
 {
+	const char *ident = service->profile;
+	GKeyFile *keyfile;
+
 	if (service->ipconfig != NULL)
 		return;
 
 	setup_ipconfig(service, index);
+
+	if (ident == NULL)
+		return;
+
+	keyfile = __connman_storage_open_profile(ident);
+	if (keyfile == NULL)
+		return;
+
+	__connman_ipconfig_load(service->ipconfig, keyfile,
+					service->identifier, "IPv4.");
+	g_key_file_free(keyfile);
 }
 
 /**
@@ -2321,7 +2388,7 @@ static void update_from_network(struct connman_service *service,
 
 	DBG("service %p network %p", service, network);
 
-	if (service->state == CONNMAN_SERVICE_STATE_READY)
+	if (is_connected(service) == TRUE)
 		return;
 
 	if (is_connecting(service) == TRUE)
